/* ----------------------------------------------------------------------
Program      : src2htm (gcc -o src2htm.exe src2htm.c)

Description  : Filter Programs and Commandline Utility

               Inspired by Emmanuel ROCHE's WS4HTM.BAS

               (The html wrapper is directly taken from
                Mr.Roche's code but this program is not
                as elegant and makes no attempt to
                "pretty-up" the HTML when processing WS files.
                Does not handle unicode characters.
                That part of Roche's HTML I did not use.
				7 bit Ascii only (strips hi-bits).)

               Converts "Unix" and Windows Text Files
               to plain-text html.

Input:  "Unix" or Windows Sequential Text File.
        Do not use Apple text files as input files.

Output: Html to Console or to File

Includes a tabstop feature which takes an optional command line argument
of the number of spaces per tabstop in the range of 1-8.

Can be used as a filter or as a standalone program. When used as a
standalone program, input and output file options can be specified as
additional command line arguments.


Written by   : Bill Buckels but based on
               Emmanuel ROCHE's WS4HTM.BAS (Mallard BASIC)

Date Revised : Nov 21, 2013

Revision     : 1.0 First Release

               Compiles under MinGW

Licence      : You may use this program for whatever you wish as long
               as you agree that Bill Buckels has no warranty or
               liability obligations whatsoever from said use.
---------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>


/*
Synopsis:
========

This is a standard tab filter that has seen multiple iterations over the
last several decades. This latest iteration has been extended to provide
html output for tab-free source listings and textfiles.

Standard Usage: Input is from stdin or a text file.
==============  Output is to stdout in html format.

Default Tabstop = 8.
Optional: Tabstop range of 1-8.

   1. Filter - pipe tabfree output to stdout or redirect to file.
	  Usage: untab
			 untab 4

   2. File Input - Open a file and output to stdout or redirect to file.
	  Usage: untab infile.txt
			 untab infile.txt 4

Special Usage: Text File Output:
=============

Default Tabstop = 4.
Optional: Tabstop range of 1-8.

   3. Convert text files with tabs to html files without tabs.

	  Usage: untab infile.txt outfile.htm
			 untab infile.txt outfile.htm 8

	  -
	  - When an output file is specified, the default tabwidth is 4.
		Optional: Tabstop range of 1-8.

 -------------------------------------------------------------------- */

char infile[512];
char outfile[512];
FILE *fp = NULL;
FILE *fp2 = NULL;

int main(int argc, char *argv[])
{
    int c,
        i,
        spaces,
        tabstop = 8, tabset = 0;    /* default to 8 space max per tab */

    /* if a valid numeric argument use the tabstop
       value that was entered on the command line */

    infile[0] = outfile[0] = 0;

    if (argc > 1) {
      for (i = 1; i < argc; i++) {
        if (argv[i][0] >= '0' && argv[i][0] <= '9') {
		  tabset = 1;
          tabstop = atoi(argv[i]);
          if (tabstop < 1 || tabstop > 8) tabstop = 8;
        }
        else {
          if (infile[0] == 0)
            strcpy(infile, argv[i]);
          else if (outfile[0] == 0)
            strcpy(outfile, argv[i]);
        }
      }
    }

    if (infile[0] != 0) {
    	fp = fopen(infile, "r");
	  	if (NULL == fp) {
		  	fprintf(stderr, "open failure on %s\n",infile);
	      	return (1);
	  	}
    }
    if (outfile[0] != 0) {
	  fp2 = fopen(outfile, "wb");

	  if (NULL == fp2) {
		  fclose(fp);
		  fprintf(stderr, "open failure on %s for write.\n",outfile);
		  return (2);
	  }
	  /* default to a tabstop of 4 */
	  if (tabset == 0) tabstop = 4;
	}

	if (NULL != fp2) {
		fprintf(fp2,"<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01//EN\">%c%c",
				0x0d,0x0a);
		fprintf(fp2,"<html>%c%c<head>%c%c",0x0d,0x0a,0x0d,0x0a);
		fprintf(fp2,"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">%c%c",
				0x0d,0x0a);
		fprintf(fp2,"<meta name=\"Generator\" content=\"src2htm by Bill Buckels\">%c%c",
				0x0d,0x0a);
		fprintf(fp2,"<title>Converted from file \"%s\"</title>\n",infile,0x0d,0x0a);
		fprintf(fp2,"</head>%c%c<basefont size=3>%c%c<body>%c%c<pre>%c%c",
				0x0d,0x0a,0x0d,0x0a,0x0d,0x0a,0x0d,0x0a);
	}
	else {
		puts("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01//EN\">");
		puts("<html>\n<head>");
		puts("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">");
		puts("<meta name=\"Generator\" content=\"src2htm by Bill Buckels\">");
		printf("<title>Converted from file \"%s\"</title>\n",infile);
		puts("</head>\n<basefont size=3>\n<body>\n<pre>");
	}


    spaces = tabstop;   /* initialize spaces to a full tab */
    for(;;)
    {
        if (NULL == fp)
          c = getchar();
        else
          c = fgetc(fp);

        if (c==EOF)break;

        if (c==0x1a) break; /* CP/M EOF */

        c &= 0x7f; /* strip hi-bits - 7 bit text only */
        /* note this should handle WS files as well as text files */

        if(c==0x0a)   /* eol found, so reset spaces to a full tabstop */
        {
          spaces = tabstop;
        }
        else
        {
          if(c==0x09) /* tab found, so print remaining spaces */
          {
            for(i=0;i<spaces;i++) {
				if (NULL == fp2) putchar(0x20);
				else fputc(0x20,fp2);
			}
            spaces = tabstop;
            continue;
          }
          spaces--;   /* other character found, one less space required
                         until next tabstop */

          if(spaces == 0)spaces = tabstop; /* spaces are all used so
                                              reset to a full tabstop */
        }
        /* replace special UTF-8 characters with HTML 2.0
           escape sequences and convert both unix and
           windows text lines by ignoring
           carriage returns from input and writing
           carriage returns before linefeeds */
        if (NULL == fp2) {
		  switch(c) {
			  case 0x22: printf("&quot;"); break;
			  case 0x26: printf("&amp;");  break;
			  case 0x3c: printf("&lt;");   break;
			  case 0x3e: printf("&gt;");   break;
              default:   putchar(c);
		  }
	    }
	    else {
		  switch(c) {
			  case 0x22: fprintf(fp2,"&quot;"); break;
			  case 0x26: fprintf(fp2,"&amp;");  break;
			  case 0x3c: fprintf(fp2,"&lt;");   break;
			  case 0x3e: fprintf(fp2,"&gt;");   break;
			  case 0x0d: break;
			  case 0x0a: fputc(0x0d, fp2);
              default: 	 fputc(c, fp2);
		  }
		}
    }

    /* print html trailer */
    if (NULL != fp2)
    	fprintf(fp2,"</pre>%c%c</body>%c%c</html>%c%c",
    	             0x0d,0x0a,0x0d,0x0a,0x0d,0x0a);
    else
    	puts("</pre>\n</body>\n</html>");


    if (NULL != fp) {
	  fclose(fp);
      if (NULL != fp2) fclose(fp2);

    }

    return (0);
}

